<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VIFEWC_Admin_Design {
	protected $settings, $capability;

	public function __construct() {
		$this->settings   = VIFEWC_DATA::get_instance();
		$this->capability = $this->settings->get_setting_capability();
		add_action( 'customize_register', array( $this, 'design_option_customizer' ) );
		add_action( 'customize_preview_init', array( $this, 'customize_preview_init' ) );
		add_action( 'customize_controls_enqueue_scripts', array( $this, 'customize_controls_enqueue_scripts' ) );
		add_action( 'wp_print_styles', array( $this, 'customize_controls_print_styles' ) );
	}

	public function design_option_customizer( $wp_customize ) {
		$this->settings = VIFEWC_DATA::get_instance( true );
		$this->customize_only_checkout_sections( $wp_customize );
		$this->customize_one_section( $wp_customize );
		$this->customize_fields( $wp_customize );
	}

	public function customize_fields( $wp_customize ) {
		$wp_customize->add_section( 'vifewc-customize-fields', array(
			'priority'       => 200,
			'capability'     => $this->capability,
			'theme_supports' => '',
			'title'          => esc_html__( '{checkout_fields} Section', 'fewc-extra-checkout-fields-for-woocommerce' ),
		) );
		$wp_customize->add_setting( 'vifewc-customize-fields', array(
			'type'              => 'vifewc-customize-fields',
			'capability'        => $this->capability,
			'sanitize_callback' => 'sanitize_text_field',
			'transport'         => 'postMessage',
		) );
		$wp_customize->add_control(
			new VIFEWC_Customize_Sections_Control(
				$wp_customize,
				'vifewc-customize-fields',
				array(
					'section' => 'vifewc-customize-fields',
				)
			)
		);
	}

	public function customize_one_section( $wp_customize ) {
		$wp_customize->add_section( 'vifewc-customize-section', array(
			'priority'       => 200,
			'capability'     => $this->capability,
			'theme_supports' => '',
			'title'          => esc_html__( 'Checkout Section', 'fewc-extra-checkout-fields-for-woocommerce' ),
		) );
		$wp_customize->add_setting( 'vifewc-customize-section', array(
			'type'              => 'vifewc-customize-section',
			'capability'        => $this->capability,
			'sanitize_callback' => 'sanitize_text_field',
			'transport'         => 'postMessage',
		) );
		$wp_customize->add_control(
			new VIFEWC_Customize_Sections_Control(
				$wp_customize,
				'vifewc-customize-section',
				array(
					'section' => 'vifewc-customize-section',
				)
			)
		);
	}

	public function customize_only_checkout_sections( $wp_customize ) {
		$wp_customize->add_section( 'vifewc', array(
			'priority'       => 200,
			'capability'     => $this->capability,
			'theme_supports' => '',
			'title'          => esc_html__( 'FEWC - Extra Checkout Fields For WooCommerce', 'fewc-extra-checkout-fields-for-woocommerce' ),
		) );
		$wp_customize->add_setting(
			new VIFEWC_Customize_Setting(
				$wp_customize,
				'vifewc_sections_params[section_id]',
				array(
					'default'           => $this->settings->get_params( 'section_id' ) ?? array(),
					'type'              => 'option',
					'capability'        => $this->capability,
					'sanitize_callback' => 'vifewc_sanitize_customizer',
					'transport'         => 'postMessage',
				)
			)
		);
		$wp_customize->add_control( 'vifewc_sections_params[section_id]', array(
			'type'     => 'hidden',
			'priority' => 10,
			'section'  => 'vifewc'
		) );
		$wp_customize->add_setting(
			new VIFEWC_Customize_Setting(
				$wp_customize,
				'vifewc_sections_params[section_settings]',
				array(
					'default'           => $this->settings->get_params( 'section_settings' ) ?? array(),
					'type'              => 'option',
					'capability'        => $this->capability,
					'sanitize_callback' => 'vifewc_sanitize_customizer',
					'transport'         => 'postMessage',
				)
			)
		);
		$wp_customize->add_control( 'vifewc_sections_params[section_settings]', array(
			'type'     => 'hidden',
			'priority' => 10,
			'section'  => 'vifewc'
		) );
		$wp_customize->add_setting(
			new VIFEWC_Customize_Setting(
				$wp_customize,
				'vifewc_sections_params[section_fields]',
				array(
					'default'           => $this->settings->get_params( 'section_fields' ) ?? array(),
					'type'              => 'option',
					'capability'        => $this->capability,
					'sanitize_callback' => 'vifewc_sanitize_customizer',
					'transport'         => 'postMessage',
				)
			)
		);
		$wp_customize->add_control( 'vifewc_sections_params[section_fields]', array(
			'type'     => 'hidden',
			'priority' => 10,
			'section'  => 'vifewc'
		) );
		$wp_customize->add_setting( 'vifewc_sections', array(
			'type'              => 'option',
			'capability'        => $this->capability,
			'sanitize_callback' => 'vifewc_sanitize_customizer',
			'transport'         => 'postMessage',
		) );
		$wp_customize->add_control(
			new VIFEWC_Customize_Sections_Control(
				$wp_customize,
				'vifewc_sections',
				array(
					'label'       => esc_html__( 'Sections', 'fewc-extra-checkout-fields-for-woocommerce' ),
					'description' => esc_html__( 'Click to edit fields for each section.', 'fewc-extra-checkout-fields-for-woocommerce' ),
					'section'     => 'vifewc',
				)
			)
		);
	}

	public function customize_controls_print_styles() {
		if ( ! is_customize_preview() ) {
			return;
		}
	}

	public function customize_controls_enqueue_scripts() {
		$this->settings::enqueue_style(
			array(
				'vifewc-customize-preview',
				'vifewc-show-message',
				'vifewc-minicolors',
				'semantic-ui-checkbox',
				'semantic-ui-icon',
				'semantic-ui-accordion',
				'semantic-ui-dropdown',
				'semantic-ui-transition'
			),
			array( 'admin-customize', 'villatheme-show-message', 'minicolors', 'checkbox', 'icon', 'accordion', 'dropdown', 'transition' ),
			array( 0, 0, 1, 1, 1, 1, 1, 1 )
		);
		$this->settings::enqueue_script(
			array( 'vifewc-minicolors', 'semantic-ui-accordion', 'semantic-ui-checkbox', 'semantic-ui-dropdown', 'semantic-ui-transition' ),
			array( 'minicolors', 'accordion', 'checkbox', 'dropdown', 'transition' ),
			array( 1, 1, 1, 1, 1 ) );
		$this->settings::enqueue_script(
			array( 'vifewc-customize-setting', 'vifewc-show-message', ),
			array( 'customize-settings', 'villatheme-show-message' ),
			array( 0, 0 ),
			array( array( 'jquery', 'jquery-ui-button', 'jquery-ui-sortable' ) ),
			'enqueue', true
		);
		$args            = array(
			'ajax_url'               => admin_url( 'admin-ajax.php' ),
			'field_validate'         => array(
				'0'        => esc_html__( 'Choose a option', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'email'    => esc_html__( 'Email', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'phone'    => esc_html__( 'Phone', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'number'   => esc_html__( 'Number', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'postcode' => esc_html__( 'Postcode', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'url'      => esc_html__( 'Url', 'fewc-extra-checkout-fields-for-woocommerce' ),
			),
			'field_as_meta'          => array(
				'order_meta' => esc_html__( 'Order meta data', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'user_meta'  => esc_html__( 'User meta data', 'fewc-extra-checkout-fields-for-woocommerce' ),
			),
			'fields_display_in'      => array(
				'admin_email'    => esc_html__( 'Admin email', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'customer_email' => esc_html__( 'Customer email', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'order_page'     => esc_html__( 'Order detail page', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'thank_you_page' => esc_html__( 'Thank you page', 'fewc-extra-checkout-fields-for-woocommerce' ),
			),
			'fields_type'            => array(
				'text'           => esc_html__( 'Text', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'email'          => esc_html__( 'Email', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'tel'            => esc_html__( 'Phone', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'number'         => esc_html__( 'Number', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'textarea'       => esc_html__( 'Textarea', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'checkbox'       => esc_html__( 'Checkbox', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'checkbox_group' => esc_html__( 'Checkbox Group', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'select'         => esc_html__( 'Select', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'radio'          => esc_html__( 'Radio', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'datetime'       => esc_html__( 'Datetime', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'password'       => esc_html__( 'Password', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'hidden'         => esc_html__( 'Hidden', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'url'            => esc_html__( 'Url', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'html'           => esc_html__( 'Html/ Paragraph', 'fewc-extra-checkout-fields-for-woocommerce' ),
			),
			'fields_time_type'       => array(
				'datetime-local' => esc_html__( 'Datetime local', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'time'           => esc_html__( 'Time', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'date'           => esc_html__( 'Date', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'week'           => esc_html__( 'Week', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'month'          => esc_html__( 'month', 'fewc-extra-checkout-fields-for-woocommerce' ),
			),
			'checkboxgroup_style'    => array(
				'vertical'   => esc_html__( 'Vertical', 'fewc-extra-checkout-fields-for-woocommerce' ),
				'horizontal' => esc_html__( 'Horizontal', 'fewc-extra-checkout-fields-for-woocommerce' ),
			),
			'fields_html_type'       => array(
				array(
					'name' => esc_html__( 'Heading', 'fewc-extra-checkout-fields-for-woocommerce' ),
					'type' => array(
						'h1' => esc_html__( 'H1', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'h2' => esc_html__( 'H2', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'h3' => esc_html__( 'H3', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'h4' => esc_html__( 'H4', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'h5' => esc_html__( 'H5', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'h6' => esc_html__( 'H6', 'fewc-extra-checkout-fields-for-woocommerce' ),
					)
				),
				array(
					'name' => esc_html__( 'Paragraph', 'fewc-extra-checkout-fields-for-woocommerce' ),
					'type' => array(
						'label' => esc_html__( 'Label', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'span'  => esc_html__( 'Span', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'p'     => esc_html__( 'P', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'div'   => esc_html__( 'div', 'fewc-extra-checkout-fields-for-woocommerce' ),
					)
				),
			),
			'section_position'       => $this->settings->section_positions(),
			'section_fields_default' => $this->settings->get_default( 'section_fields' ),
			'enable'                 => 1,
			'home_url'               => esc_js( home_url() ),
			'checkout_url'           => esc_js( wc_get_page_permalink( 'checkout' ) )
		);
		$section_disable = array();
		if ( ! wc_shipping_enabled() || wc_ship_to_billing_address_only() ) {
			$section_disable[] = 'shipping';
		}
		$args['section_disable'] = $section_disable;
		$text                    = array(
			/* translators: %1s: Method name */
			'deactivate_warning'              => wp_kses_post( sprintf( __( 'All features are not working on your site unless the customize. Please turn on the option <a class="vifewc-warning" target="_blank" href="%1s">Enable</a> to use that.',
				'fewc-extra-checkout-fields-for-woocommerce' ), esc_url( admin_url( 'admin.php?page=vifewc' ) ) ) ),
			'wrap_class_desc'                 => esc_html__( 'Please enter some text, or some class by "," separating value. You can use one of these class: form-row-first, form-row-last, form-row-wide',
				'fewc-extra-checkout-fields-for-woocommerce' ),
			'general_settings_title'          => esc_html__( 'General', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_name'                    => esc_html__( 'Name', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_title'                   => esc_html__( 'Title', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_title_color'             => esc_html__( 'Title Color', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_position'                => esc_html__( 'Position', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_action_edit_fields'      => esc_html__( 'Manage Section fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_action_delete'           => esc_html__( 'Remove This Section', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_remove'                  => esc_html__( 'Are you sure to remove this section?', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'section_fields_reset'            => esc_html__( 'Reset Default', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_info_title'                => esc_html__( 'Field Detail', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_general_title'             => esc_html__( 'General', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_design_title'              => esc_html__( 'Design', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_data_title'                => esc_html__( 'Meta Data', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_enable'              => esc_html__( 'Enable', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_required'            => esc_html__( 'Required', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_label'               => esc_html__( 'Label', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_type'                => esc_html__( 'Type', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_class'               => esc_html__( 'Class', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_label_class'         => esc_html__( 'Label Class', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_input_class'         => esc_html__( 'Input Class', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_save_as'             => esc_html__( 'Save As', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_meta_key'            => esc_html__( 'Meta Key Name', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_display'             => esc_html__( 'Display In', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_desc'                => esc_html__( 'Description', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_placeholder'         => esc_html__( 'Placeholder', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_default'             => esc_html__( 'Default Value', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_minlength'           => esc_html__( 'Minimum Value Length', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_maxlength'           => esc_html__( 'Maximum Value Length', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_step'                => esc_html__( 'Step', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_minvalue'            => esc_html__( 'Minimum Value', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_maxvalue'            => esc_html__( 'Maximum Value', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_checked_default'     => esc_html__( 'Set checkbox state "checked" as default', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_selected_default'    => esc_html__( 'Is the default selected value', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_checkboxgroup_style' => esc_html__( 'Style', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_options'             => esc_html__( 'Options', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_option_value'        => esc_html__( 'Option value(*)', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_option_label'        => esc_html__( 'Option label(*)', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_option_remove'             => esc_html__( 'Are you sure to remove this option?', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_content'             => esc_html__( 'Content', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_title_validation'          => esc_html__( 'Validations', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'field_remove'                    => esc_html__( 'Are you sure to remove this field?', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_remove_field'              => esc_html__( 'You can not remove this field', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_remove_last_item'          => esc_html__( 'You can not remove the last item.', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_option_value_invalid'      => esc_html__( 'The option value is invalid', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_option_value_unique'       => esc_html__( 'The option value must be unique!', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_option_value_empty'        => esc_html__( 'The option value can not be empty!', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_option_label_empty'        => esc_html__( 'The option label can not be empty!', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_name_empty'                => esc_html__( 'Name can not be empty!', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'error_name_unique'               => esc_html__( 'Name must be unique!', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'save_section'                    => esc_html__( 'Save section', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'add_section'                     => esc_html__( 'Add section', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'enable'                          => esc_html__( 'Enable', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'disable'                         => esc_html__( 'Disable', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'save'                            => esc_html__( 'Apply', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'edit'                            => esc_html__( 'Edit', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'clone'                           => esc_html__( 'Clone', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'remove'                          => esc_html__( 'Remove', 'fewc-extra-checkout-fields-for-woocommerce' ),
		);
		wp_localize_script( 'vifewc-customize-setting', 'vifewc_preview_setting', $args );
		wp_localize_script( 'vifewc-customize-setting', 'vifewc_preview_text', $text );
	}

	public function customize_preview_init() {
		$this->settings::enqueue_script(
			array( 'vifewc-customize-preview' ),
			array( 'customize-preview' ),
			array( 0 ),
			array( array( 'jquery', 'customize-preview' ) ),
			'enqueue', true
		);
		$args = array(
			'ajax_url' => admin_url( 'admin-ajax.php' )
		);
		wp_localize_script( 'vifewc-customize-preview', 'vifewc_preview', $args );
	}
}

?>