<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VIFEWC_DATA {
	private $params;
	private $default;
	private static $allow_html;
	protected static $instance = null;

	/**
	 * VIFEWC_DATA constructor.
	 */
	public function __construct() {
		global $vifewc_settings, $vifewc_export;
		if ( ! $vifewc_settings ) {
			$general1         = get_option( 'vifewc_settings_params', array() );
			$custom_sections1 = get_option( 'vifewc_sections_params', array() );
			$vifewc_settings  = array_merge( $general1, $custom_sections1 );
		}
		$general         = array(
			'enable' => ''
		);
		$custom_sections = array(
			'section_id'       => array( 'billing', 'shipping', 'order' ),
			'section_settings' => array(
				'billing'  => array(
					'name' => 'Billing'
				),
				'shipping' => array(
					'name' => 'Shipping'
				),
				'order'    => array(
					'name' => 'Additional Information'
				)
			),
			'section_fields'   => array(
				'billing'  => $this->get_default_fields(),
				'shipping' => $this->get_default_fields( 'shipping_' ),
				'order'    => array(
					'order_comments' => array(
						'active'      => 1,
						'type'        => 'textarea',
						'class'       => array( 'notes' ),
						'label'       => esc_html__( 'Order notes', 'fewc-extra-checkout-fields-for-woocommerce' ),
						'placeholder' => esc_attr__( 'Notes about your order, e.g. special notes for delivery.', 'fewc-extra-checkout-fields-for-woocommerce' ),
					),
				)
			),
		);
		$this->default   = array_merge( $general, $custom_sections );
		$this->params    = apply_filters( 'vifewc_params', wp_parse_args( $vifewc_settings, $this->default ) );

		$vifewc_export = array(
			'general' => wp_parse_args(get_option( 'vifewc_settings_params', array() ),$general ),
			'sections' => wp_parse_args( get_option( 'vifewc_sections_params', array() ), $custom_sections)
		);
	}

	public function get_default_fields( $type = 'billing_' ) {
		if ( ! function_exists( 'WC' ) || empty(WC()->countries) ) {
			return null;
		}
		$country = WC()->countries->get_base_country();
		if ( $type === 'billing_' ) {
			$allowed_countries = array_keys( WC()->countries->get_allowed_countries() );
		} else {
			$allowed_countries = array_keys( WC()->countries->get_shipping_countries() );
		}
		if ( ! in_array( $country, $allowed_countries ) ) {
			$country = current( $allowed_countries );
		}

		return WC()->countries->get_address_fields( $country, $type );
	}

	public static function get_section_fields( $section_fields, $fields, $section_id ) {
		if ( empty( $section_fields ) && ! is_array( $section_fields ) ) {
			return $fields;
		}
		$result = array();
		global $viwcaio_get_checkout_form;
		if ( $viwcaio_get_checkout_form ) {
			$edit_address = false;
		} else {
			$edit_address = is_wc_endpoint_url( 'edit-address' ) && in_array( $section_id, [ 'billing', 'shipping' ] );
		}
		foreach ( $section_fields as $id => $data ) {
			$enable = $data['enable'] ?? 1;
			if ( ( ! $enable && ! $edit_address ) || ( ! $enable && $edit_address && ! in_array( $id, [ 'billing_country', 'shipping_country' ] ) ) ||
			     ! apply_filters( "vifewc_enable_section_field", true, $id, $data ) ) {
				continue;
			}
			if ( ! isset( $data['class'] ) ) {
				$data['class'] = array();
			}
			if (in_array($id,['billing_city','billing_state','shipping_city','shipping_state'])){
				$data['type'] = $fields[$id]['type'] ??'';
			}
			$data['class'][] = 'vifewc-field';
			$data['class'][] = 'vifewc-field-' . ( $data['type'] ?? 'text' );
			if (in_array($id,['billing_state','shipping_state'])){
				$data['required'] = $fields[$id]['required'] ??'';
			}
			if ( ! empty( $data['required'] ) ) {
				$data['class'][] = 'vifewc-validate-required';
			}
			if ( $edit_address && !empty( $data['is_custom'] )) {
				if ( ( $data['type'] ?? '' ) === 'html' ) {
					continue;
				}
				$save_as = $data['save_as'] ?? array();
				if ( ! is_array( $save_as ) || ! in_array( 'user_meta', $save_as ) ) {
					continue;
				}
			}
			if ( ! isset( $data['custom_attributes'] ) ) {
				$data['custom_attributes'] = array();
			}
			$attributes = array( 'minlength', 'min', 'max', 'step' );
			foreach ( $attributes as $item ) {
				if ( ! empty( $data[ $item ] ) ) {
					$data['custom_attributes'][ $item ] = $data[ $item ];
				}
			}
			$data['vifewc_field']=1;
			if ( ! empty( $data['meta_key'] ) ) {
				$data['id']                  = $id;
				$result[ $data['meta_key'] ] = $data;
			} else {
				$result[ $id ] = $data;
			}
		}
		return $result;
	}

	/**
	 * @param bool $new
	 *
	 * @return VIFEWC_DATA|null
	 */
	public static function get_instance( $new = false ) {
		if ( $new || null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * @param string $name
	 *
	 * @return bool|mixed|void
	 */
	public function get_params( $name = '' ) {
		if ( ! $name ) {
			return $this->params;
		}
		if ( isset( $this->params[ $name ] ) ) {
			return apply_filters( 'vifewc_params_' . $name, $this->params[ $name ] );
		}

		return null;
	}



	public function get_current_setting_by_subtitle( $name = "", $subtitle = "", $i = 0, $default = false ) {
		if ( empty( $name ) ) {
			return false;
		}
		if ( $default !== false ) {
			$result = $this->get_current_setting( $name, $subtitle )[ $i ] ?? $default;
		} else {
			$result = $this->get_current_setting( $name, $subtitle )[ $i ] ?? false;
		}

		return $result;
	}

	public function get_current_setting( $name = "", $i = 0, $default = null ) {
		if ( ! $name ) {
			return false;
		}
		if ( $default !== null ) {
			$result = $this->get_params( $name )[ $i ] ?? $default;
		} else {
			$result = $this->get_params( $name )[ $i ] ?? $this->get_default( $name )[ $i ] ?? false;
		}

		return $result;
	}

	public function get_default( $name = "" ) {
		if ( ! $name ) {
			return $this->default;
		} elseif ( isset( $this->default[ $name ] ) ) {
			return apply_filters( 'vifewc_params_default_' . $name, $this->default[ $name ] );
		} else {
			return false;
		}
	}

	public function section_positions() {
		 $arg = array(
			'woocommerce_checkout_before_customer_details'     => esc_html__( 'Before customer details', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_checkout_after_customer_details'      => esc_html__( 'After customer details', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_before_checkout_billing_form'         => esc_html__( 'Before billing form fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_after_checkout_billing_form'          => esc_html__( 'After billing form fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_before_checkout_registration_form'    => esc_html__( 'Before registration form fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_after_checkout_registration_form'     => esc_html__( 'After registration form fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_before_checkout_shipping_form'        => esc_html__( 'Before shipping form fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_after_checkout_shipping_form'         => esc_html__( 'After shipping form fields', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_before_order_notes'                   => esc_html__( 'Before order notes', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_after_order_notes'                    => esc_html__( 'After order notes', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_checkout_before_order_review'         => esc_html__( 'Before order review', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_checkout_after_order_review'          => esc_html__( 'After order review', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_review_order_before_cart_contents'    => esc_html__( 'Before cart contents', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_review_order_after_cart_contents'     => esc_html__( 'After cart contents', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_review_order_before_order_total'      => esc_html__( 'Before order total', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_review_order_after_order_total'       => esc_html__( 'After order total', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_checkout_before_terms_and_conditions' => esc_html__( 'Before terms and conditions', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_checkout_after_terms_and_conditions'  => esc_html__( 'After terms and conditions', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_review_order_before_submit'           => esc_html__( 'Before place order button', 'fewc-extra-checkout-fields-for-woocommerce' ),
			'woocommerce_review_order_after_submit'            => esc_html__( 'After place order button', 'fewc-extra-checkout-fields-for-woocommerce' ),
		);
		if (!apply_filters( 'woocommerce_checkout_registration_enabled', 'yes' === get_option( 'woocommerce_enable_signup_and_login_from_checkout' ) )){
			unset($arg['woocommerce_before_checkout_registration_form']);
			unset($arg['woocommerce_after_checkout_registration_form']);
		}
		return apply_filters('vifewc_get_section_position',$arg);
	}

	public function get_option_autoload() {
		return apply_filters( 'vifewc_option_autoload', 'no' );
	}

	public function get_setting_capability() {
		return apply_filters( 'vifewc_setting_capability', 'manage_woocommerce' );
	}

	public static function filter_allowed_html( $tags ) {
		$tags = array_merge_recursive( $tags, array(
				'input'  => array(
					'type'         => 1,
					'id'           => 1,
					'name'         => 1,
					'class'        => 1,
					'placeholder'  => 1,
					'autocomplete' => 1,
					'style'        => 1,
					'value'        => 1,
					'size'         => 1,
					'checked'      => 1,
					'disabled'     => 1,
					'readonly'     => 1,
					'data-*'       => 1,
				),
				'form'   => array(
					'method' => 1,
					'id'     => 1,
					'class'  => 1,
					'action' => 1,
					'data-*' => 1,
				),
				'select' => array(
					'id'       => 1,
					'name'     => 1,
					'class'    => 1,
					'multiple' => 1,
					'data-*'   => 1,
				),
				'option' => array(
					'value'    => 1,
					'selected' => 1,
					'data-*'   => 1,
				),
				'style'  => array(
					'id'    => 1,
					'class' => 1,
					'type'  => 1,
				),
			)
		);
		foreach ( $tags as $key => $value ) {
			if ( $key === 'input' ) {
				$tags[ $key ]['data-*']   = 1;
				$tags[ $key ]['checked']  = 1;
				$tags[ $key ]['disabled'] = 1;
				$tags[ $key ]['readonly'] = 1;
			} elseif ( in_array( $key, array( 'div', 'span', 'a', 'form', 'select', 'option', 'tr', 'td' ) ) ) {
				$tags[ $key ]['data-*'] = 1;
			}
		}

		return $tags;
	}

	public static function wp_kses_post( $content ) {
		if ( self::$allow_html === null ) {
			self::$allow_html = wp_kses_allowed_html( 'post' );
			self::$allow_html = self::filter_allowed_html( self::$allow_html );
		}

		return wp_kses( $content, self::$allow_html );
	}


	public static function remove_other_script() {
		global $wp_scripts;
		if ( isset( $wp_scripts->registered['jquery-ui-accordion'] ) ) {
			unset( $wp_scripts->registered['jquery-ui-accordion'] );
			wp_dequeue_script( 'jquery-ui-accordion' );
		}
		if ( isset( $wp_scripts->registered['accordion'] ) ) {
			unset( $wp_scripts->registered['accordion'] );
			wp_dequeue_script( 'accordion' );
		}
		$scripts = $wp_scripts->registered;
		foreach ( $scripts as $k => $script ) {
			if ( in_array( $script->handle, array( 'query-monitor', 'uip-app', 'uip-vue', 'uip-toolbar-app' ) ) ) {
				continue;
			}
			preg_match( '/\/wp-/i', $script->src, $result );
			if ( count( array_filter( $result ) ) ) {
				preg_match( '/(\/wp-content\/plugins|\/wp-content\/themes)/i', $script->src, $result1 );
				if ( count( array_filter( $result1 ) ) ) {
					wp_dequeue_script( $script->handle );
				}
			} else {
				wp_dequeue_script( $script->handle );
			}
		}
	}

	public static function enqueue_style( $handles = array(), $srcs = array(), $is_suffix = array(), $des = array(), $type = 'enqueue' ) {
		if ( empty( $handles ) || empty( $srcs ) ) {
			return;
		}
		$action = $type === 'enqueue' ? 'wp_enqueue_style' : 'wp_register_style';
		$suffix = WP_DEBUG ? '' : '.min';
		foreach ( $handles as $i => $handle ) {
			if ( ! $handle || empty( $srcs[ $i ] ) ) {
				continue;
			}
			$suffix_t = ! empty( $is_suffix[ $i ] ) ? '.min' : $suffix;
			$action( $handle, VIFEWC_CSS . $srcs[ $i ] . $suffix_t . '.css', ! empty( $des[ $i ] ) ? $des[ $i ] : array(), VIFEWC_VERSION );
		}
	}

	public static function enqueue_script( $handles = array(), $srcs = array(), $is_suffix = array(), $des = array(), $type = 'enqueue', $in_footer = false ) {
		if ( empty( $handles ) || empty( $srcs ) ) {
			return;
		}
		$action = $type === 'register' ? 'wp_register_script' : 'wp_enqueue_script';
		$suffix = WP_DEBUG ? '' : '.min';
		foreach ( $handles as $i => $handle ) {
			if ( ! $handle || empty( $srcs[ $i ] ) ) {
				continue;
			}
			$suffix_t = ! empty( $is_suffix[ $i ] ) ? '.min' : $suffix;
			$action( $handle, VIFEWC_JS . $srcs[ $i ] . $suffix_t . '.js', ! empty( $des[ $i ] ) ? $des[ $i ] : array( 'jquery' ), VIFEWC_VERSION, $in_footer );
		}
	}
}